/** This file is part of amemiya.
 *
 *  amemiya is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  amemiya is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with amemiya.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "AM_ArrayList.h"

#define ARRAYLIST_INITIAL_CAPACITY 10

static void * mallocChecker(const size_t size);

void am_freeList(const AM_ArrayList list){
	if(list->data!=NULL){
		free(list->data);
	}
	if(list!=NULL){
		free(list);
	}
}

int am_listAdd(const AM_ArrayList list, void *object){
	int oldSize = list->size;
	int newCapacity;
	void **newData;
	(list->size)++;
	if(oldSize == list->currentCapacity){
		newCapacity = list->currentCapacity+10;
		newData = mallocChecker(sizeof(void *) * newCapacity);
		memcpy(newData, list->data, (size_t)(sizeof(void *) * oldSize));
		free(list->data);
		(list->data) = newData;
		list->currentCapacity = newCapacity;
	}
	(list->data)[oldSize] = object;
	return 1;
}

int am_listRemove(const AM_ArrayList list, const int index){
	int length = list->size;
	int lastIndex = length - 1;
	int newSize, newCapacity;
	(list->size)--;
	if(index == lastIndex){
		return 1;
	}else if(index >= 0 && index < lastIndex){
		memmove(list->data + index, list->data + index + 1, (size_t)(sizeof(void *) * (lastIndex - index)));
		newSize = list->size;
		newCapacity = list->currentCapacity - 10;
		if(newCapacity > newSize){
			list->data = realloc(list->data, sizeof(void *) * newCapacity);
			list->currentCapacity = newCapacity;
		}
		return 1;
	}
	return 0; //failure
}

void * am_listGet(const AM_ArrayList list, const int index){
	if(index >= 0 && index < list->size){
		return list->data[index];
	}
	return NULL;
}

static void * mallocChecker(const size_t size){
	void *data;
	data = malloc(size);
	if(data == NULL){
		fprintf(stderr, "Out of memory\n");
		fflush(stderr);
		exit(0);
	}
	return data;
}

void am_listClear(const AM_ArrayList list){
	list->data = realloc(list->data, sizeof(void *) * ARRAYLIST_INITIAL_CAPACITY);
	list->currentCapacity = ARRAYLIST_INITIAL_CAPACITY;
	list->size = 0;
}

AM_ArrayList am_newArrayList(){
	AM_ArrayList list;
	list = mallocChecker(sizeof(struct AM_ArrayList_Struct));
	list->currentCapacity = ARRAYLIST_INITIAL_CAPACITY;
	list->data = mallocChecker(sizeof(void *) * list->currentCapacity);
	list->size = 0;
	return list;
}
