/** This file is part of amemiya.
 *
 *  amemiya is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  amemiya is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with amemiya.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

#include "AM_Ffmpeg.h"

static const int64_t FIRST_FRAME = 1;

void am_initAV(AVFormatContext **fc, const char *videoFile){
	av_register_all();
	if(av_open_input_file(fc, videoFile, NULL, 0, NULL)!=0)
		exit(-1);
	if(av_find_stream_info(*fc)<0)
        exit(-1);
    //if(debug == 1){
    //	dump_format(*fc, 0, videoFile, 0);
    //}
}

void am_printCodecString(AVCodecContext *avcc){
	char buf[256];
	avcodec_string(buf,sizeof(buf),avcc,0);
	fprintf(stderr,"codec string = %s\n",buf);
}

AM_Ffmpeg am_openVideo(char *videoFile, const char *outFile,
		const int resizeWidth, const int resizeHeight,
		const int readChoice,
		const AM_ArrayList videoTrims,
		const AM_ArrayList audioTrims){
	AM_Ffmpeg c;
	c = malloc(sizeof(struct AM_Ffmpeg_Struct));
	if(c == NULL){
		fprintf(stderr, "Out of memory.\n");
		exit(0);
	}
	c->readChoice = readChoice;
	c->videoFile = videoFile;

	c->vc = malloc(sizeof(struct AM_VideoContext_Struct));
	if(c->vc == NULL){
		fprintf(stderr, "Out of memory.\n");
		exit(0);
	}
	c->vc->swsSetupBool=0;
	c->vc->v = am_newVideoFrame();

	c->ac = malloc(sizeof(struct AM_AudioContext_Struct));
	if(c->ac == NULL){
		fprintf(stderr, "Out of memory.\n");
		exit(0);
	}

	am_initAV(&(c->fc),videoFile);
	c->vc->trims=videoTrims;
	c->ac->trims=audioTrims;
	c->vc->stream=-1;
	c->ac->stream=-1;
	int i=0;
    for(i=0; i<c->fc->nb_streams; i++){
        if(c->fc->streams[i]->codec->codec_type==CODEC_TYPE_VIDEO){
            c->vc->stream=i;
        }
		if(c->fc->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO){
            c->ac->stream=i;
        }
	}

    if(c->vc->stream==-1)
        return NULL;
	if(c->ac->stream==-1)
		return NULL;

	c->vc->fps=c->fc->streams[c->vc->stream]->r_frame_rate.num / (double)c->fc->streams[c->vc->stream]->r_frame_rate.den;
	c->vc->codecCtx=c->fc->streams[c->vc->stream]->codec;
	c->ac->codecCtx=c->fc->streams[c->ac->stream]->codec;

    c->vc->codec=avcodec_find_decoder(c->vc->codecCtx->codec_id);
    if(c->vc->codec==NULL)
        return NULL;
	c->ac->codec=avcodec_find_decoder(c->ac->codecCtx->codec_id);
	if(c->ac->codec==NULL)
		return NULL;
    if(!c->vc->codec || (avcodec_open(c->vc->codecCtx, c->vc->codec)<0))
        return NULL;
	if(!c->ac->codec || (avcodec_open(c->ac->codecCtx, c->ac->codec)<0))
		return NULL;

	c->ac->audHz = c->ac->codecCtx->sample_rate;
	c->ac->audChannels = c->ac->codecCtx->channels;
	if(c->ac->audChannels==1){
		c->ac->audChannels=2;
	}

	if(resizeWidth==0 || resizeHeight==0){
		c->vc->resizeWidth=0;
		c->vc->resizeHeight=0;
		c->vc->v->width=c->vc->codecCtx->width;
		c->vc->v->height=c->vc->codecCtx->height;
		c->vc->resizeBool=0;
	}else{
		c->vc->resizeBool=1;
		c->vc->resizeWidth=resizeWidth;
		c->vc->resizeHeight=resizeHeight;
		c->vc->v->width=c->vc->resizeWidth;
		c->vc->v->height=c->vc->resizeHeight;
	}

	if(c->vc->resizeWidth==c->vc->codecCtx->width && c->vc->resizeHeight==c->vc->codecCtx->height){
		c->vc->resizeBool=0;
	}

	// Hack to correct wrong frame rates that seem to be generated by some codecs
    if(c->vc->codecCtx->time_base.num>1000 && c->vc->codecCtx->time_base.den==1)
		c->vc->codecCtx->time_base.den=1000;

    c->vc->frame = avcodec_alloc_frame();
	if(c->vc->frame==NULL){
		return NULL;
	}
	c->ac->frame = malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);

	c->vc->resizeFrame=avcodec_alloc_frame();
    if(c->vc->resizeFrame==NULL)
        return NULL;
	c->vc->numBytes=avpicture_get_size(c->vc->codecCtx->pix_fmt, c->vc->resizeWidth,
								c->vc->resizeHeight);
	c->vc->buffer=0;
	if(c->vc->numBytes>=0){
		c->vc->buffer=malloc(c->vc->numBytes);
	}

	avpicture_fill((AVPicture *)(c->vc->resizeFrame), c->vc->buffer, c->vc->codecCtx->pix_fmt,
				   c->vc->resizeWidth, c->vc->resizeHeight);

	/*if(debug == 1){
		printCodecString(c->vc->codecCtx);
		printCodecString(c->ac->codecCtx);
		fprintf(stderr,"audio sample rate = %d\n", c->ac->codecCtx->sample_rate);
		fprintf(stderr,"audio channels = %d\n", c->ac->codecCtx->channels);
		fprintf(stderr,"audio bit rate = %d\n",c->ac->codecCtx->bit_rate);
		outFormatStuff(c->fc);
	}*/

	c->out = fopen(outFile,"wb");
	if(c->out==NULL){
		return NULL;
	}
	c->ac->frameCount=FIRST_FRAME-1;
	c->vc->frameCount=FIRST_FRAME-1;
	c->ac->trimIterator=0;
	c->vc->trimIterator=0;
	c->vc->videoDone=0;
	c->ac->audioDone=0;
	c->decodePacketFreed=1;
	return c;
}

void am_closeVideo(const AM_Ffmpeg c){
	if(c->out!=NULL){
		fclose(c->out);
	}

	if(c->vc->buffer!=NULL){
		free(c->vc->buffer);
	}
	if(c->vc->resizeFrame!=NULL){
		av_free(c->vc->resizeFrame);
	}
	if(c->vc->frame!=NULL){
		av_free(c->vc->frame);
	}
	if(c->ac->frame!=NULL){
		free(c->ac->frame);
	}
	if(c->ac->codecCtx!=NULL){
		avcodec_close(c->ac->codecCtx);
	}

	int tt = 0;
	for(tt=0; tt<c->ac->trims->size; tt++){
		am_freeTrim((const AM_Trim)am_listGet(c->ac->trims,tt));
	}
	am_freeList(c->ac->trims);
	if(c->ac!=NULL){
		free(c->ac);
	}
	if(c->readChoice!=4||c->vc->resizeWidth>0||c->vc->resizeHeight>0){
		if(c->vc->swsSetupBool==1){
			if(c->vc->sws!=NULL){
				sws_freeContext(c->vc->sws);
			}
		}
	}
	if(c->vc->codecCtx!=NULL){
		avcodec_close(c->vc->codecCtx);
	}
    if(c->vc->v!=NULL){
    	am_freeVideoFrame(c->vc->v);
    }
	for(tt=0; tt<c->vc->trims->size; tt++){
		am_freeTrim((const AM_Trim)am_listGet(c->vc->trims,tt));
	}
	am_freeList(c->vc->trims);
	if(c->vc!=NULL){
		free(c->vc);
	}
	// Close video file
	if(c->fc!=NULL){
		av_close_input_file(c->fc);
	}
	if(c!=NULL){
		free(c);
	}
}

AM_Boolean am_getNextFrame(const AM_Ffmpeg c){
	if(c->vc->videoDone==1||c->ac->audioDone==1){
		return false;
	}
	while(av_read_frame(c->fc, &(c->decodingPacket))>=0){
		if(c->readChoice == 1 || c->readChoice == 4){
		    if(am_decodeVideoFrame(c)){
		    	c->vc->frameCount++;
		    	avFreePacket(&(c->decodingPacket));
		    	return true;
		    }
		}
		if(c->readChoice == 2 || c->readChoice == 4){
			if(am_decodeAudioFrame(c)){
				c->ac->frameCount++;
				avFreePacket(&(c->decodingPacket));
				return true;
			}
		}
	}
	return false;
}

AM_Boolean am_readVideo(const AM_Ffmpeg c){
    while(am_getNextFrame(c)){
    	if(c->readChoice==1){
    		am_fprintVideo(c);
    	}else if(c->readChoice==2){
    		am_fprintAudio(c);
    	}
    }
    if(c->readChoice == 4){
    	char tmp[1024];
    	sprintf(tmp,"%s.txt",c->videoFile);
    	FILE *data = fopen(tmp,"w");
    	if(data==NULL){
    		return false;
    	}
    	fprintf(data,"width=%d\nheight=%d\nfps=%2.2f\naudHz=%d\naudChannels=%d\naudmaxbitrate=%d\n",c->vc->v->width,c->vc->v->height,c->vc->fps,c->ac->audHz, c->ac->audChannels, c->ac->codecCtx->bit_rate);
    	fprintf(data,"audframesize=%d\n",c->ac->frameSize);
    	fprintf(data,"videoframes=%lld\n",c->vc->frameCount);
    	fprintf(data,"audioframes=%lld\n",c->ac->frameCount);
    	fflush(data);
    	fclose(data);
    }
    return true;
}

void am_setUpFrame(const AM_Ffmpeg c){
	am_resizeFrame(c->vc);
	c->vc->v->picType=c->vc->frame->pict_type;
	c->vc->v->interlacedFrame=c->vc->frame->interlaced_frame;
	c->vc->v->topFieldFirst=c->vc->frame->top_field_first;
	/*printf("interlaced = %d\n", vc->v->isInterlaced(vc->v));
	printf("top field first = %d\n", vc->v->isTopFieldFirst(vc->v));*/
}

/**
 * Decodes a video frame
 * param 	c An AM_Ffmpeg structure
 * return 	true upon the frame being decoded, false otherwise.
 */
AM_Boolean am_decodeVideoFrame(const AM_Ffmpeg c){
	if(c->decodingPacket.stream_index==c->vc->stream){
		avcodec_decode_video2(c->vc->codecCtx, c->vc->frame, &(c->vc->frameFinished), &(c->decodingPacket));
		if(c->vc->frameFinished){
			return true;
		}
		return false;
	}
	return false;
}

void am_fprintVideo(const AM_Ffmpeg c){
	if(c->readChoice == 4){
	}else if(c->vc->trims->size>0){
		if(am_insideTrim((const AM_Trim)am_listGet(c->vc->trims,c->vc->trimIterator),c->vc->frameCount)==1){
			am_setUpFrame(c);
			am_printVideoFrame(c->vc->v,c->out);
			if(am_atTrimEnd((const AM_Trim)am_listGet(c->vc->trims,c->vc->trimIterator),c->vc->frameCount)==1 && c->vc->trimIterator+1 < c->vc->trims->size){
				c->vc->trimIterator+=1;
				if(c->vc->frameCount+1>(((AM_Trim)am_listGet(c->vc->trims,c->vc->trimIterator))->start)){
					am_seekToFirstFrame(c->fc,c->vc->stream);
					c->vc->frameCount=FIRST_FRAME-1;
				}
			}else if(am_atTrimEnd((const AM_Trim)am_listGet(c->vc->trims,c->vc->trimIterator),c->vc->frameCount)==1 && c->vc->trimIterator+1 == c->vc->trims->size){
				c->vc->videoDone=1;
			}
		}
	}else{
		am_setUpFrame(c);
		am_printVideoFrame(c->vc->v,c->out);
	}
}

/**
 * Decodes an audio frame
 * param 	c 	An AM_Ffmpeg structure
 * return 	true upon the frame being decoded, false otherwise.
 */
AM_Boolean am_decodeAudioFrame(const AM_Ffmpeg c){
	if(c->decodingPacket.stream_index==c->ac->stream){
		c->ac->frameSize = AVCODEC_MAX_AUDIO_FRAME_SIZE;
		int len = avcodec_decode_audio3(c->ac->codecCtx, (int16_t *)c->ac->frame, &(c->ac->frameSize), &(c->decodingPacket));
		if(len < 0){
			fprintf(stderr, "Error while decoding\n");
			return false;
		}
		if(c->ac->frameSize > 0){
			return true;
		}
		return false;
	}
	return false;
}

void am_fprintAudio(const AM_Ffmpeg c){
	if(c->ac->frameCount==0){
		c->ac->bytesPerFrame=c->ac->frameSize;
		//fprintf(stderr,"audio start pts:dts = %lld:%lld\n",packet.pts,packet.dts);
	}
	if(c->readChoice == 4){
	}else if(c->ac->trims->size>0){
		if(am_insideTrim((const AM_Trim)am_listGet(c->ac->trims,c->ac->trimIterator),c->ac->frameCount)==1){
			am_printAudio(c->ac->frame,c->ac->frameSize,c->out);
			if(am_atTrimEnd((const AM_Trim)am_listGet(c->ac->trims,c->ac->trimIterator),c->ac->frameCount)==1 && c->ac->trimIterator+1 < c->ac->trims->size){
				c->ac->trimIterator+=1;
				if(c->ac->frameCount+1>(((AM_Trim)am_listGet(c->ac->trims,c->ac->trimIterator))->start)){
					am_seekToFirstFrame(c->fc,c->ac->stream);
					c->ac->frameCount=FIRST_FRAME-1;
				}
			}else if(am_atTrimEnd((const AM_Trim)am_listGet(c->ac->trims,c->ac->trimIterator),c->ac->frameCount)==1 && c->ac->trimIterator+1 == c->ac->trims->size){
				c->ac->audioDone=1;
			}
		}
	}else{
		am_printAudio(c->ac->frame,c->ac->frameSize,c->out);
	}
}

void am_seekToFirstFrame(AVFormatContext *fc, const int streamIndex){
	av_seek_frame(fc, streamIndex, 0, 0 );
}

void am_resizeFrame(const AM_VideoContext vc){
	if(vc->resizeBool==1){
		if(vc->swsSetupBool!=1){
			int w = vc->codecCtx->width;
			int h = vc->codecCtx->height;
			int fmt = vc->codecCtx->pix_fmt;
			//if(debug == 1){
			//	fprintf(stderr,"w=%d, h=%d, rw=%d, rh=%d, fmt=%d\n",w,h,vc->resizeWidth,vc->resizeHeight,fmt);
			//}
			vc->sws = sws_getContext(w,h,fmt,vc->resizeWidth,vc->resizeHeight,fmt,SWS_LANCZOS,NULL,NULL,NULL);
			if(vc->sws == NULL){
				fprintf(stderr, "Cannot initialize the conversion context!\n");
				exit(1);
			}
			vc->swsSetupBool=1;
		}

		int ret = sws_scale(vc->sws,
				(const uint8_t* const*)vc->frame->data,
				vc->frame->linesize,
				0, vc->codecCtx->height,
				vc->resizeFrame->data,
				vc->resizeFrame->linesize);
		if(ret){
		}
		int i=0;
		for(i=0; i<4; i++){
			vc->v->data[i]=vc->resizeFrame->data[i];
			vc->v->linesize[i]=vc->resizeFrame->linesize[i];
		}
	}else{
		int i=0;
		for(i=0; i<4; i++){
			vc->v->data[i]=vc->frame->data[i];
			vc->v->linesize[i]=vc->frame->linesize[i];
		}
	}
}

void avFreePacket(AVPacket *pkt){
	if(pkt){
		if(pkt->destruct) pkt->destruct(pkt);
		pkt->data = NULL; pkt->size = 0;
	}
}

int am_printAudio(uint8_t *outbuf, int outsize, FILE *outFile){
	//fprintf(stderr,"%d\n",outsize);
	fwrite(outbuf,1,outsize,outFile);
	return 1;
}


/*void cropFrame(){
	if (ost->video_crop) {
		if (av_picture_crop((AVPicture *)&picture_crop_temp, (AVPicture *)in_picture, dec->pix_fmt, ost->topBand, ost->leftBand) < 0) {
			av_log(NULL, AV_LOG_ERROR, "error cropping picture\n");
			if (exit_on_error)
				av_exit(1);
			return;
		}
		formatted_picture = &picture_crop_temp;
	} else {
		formatted_picture = in_picture;
	}
}

void padFrame(const AM_VideoContext vc){
	if (vc->videoPad) {
		final_picture = &ost->pict_tmp;
		if (ost->video_resample) {
			if (av_picture_crop((AVPicture *)&picture_pad_temp, (AVPicture *)final_picture, enc->pix_fmt, ost->padtop, ost->padleft) < 0) {
				av_log(NULL, AV_LOG_ERROR, "error padding picture\n");
				if (exit_on_error)
					av_exit(1);
				return;
			}
			resampling_dst = &picture_pad_temp;
		}
	}
}*/
