/** This file is part of amemiya.
 *
 *  amemiya is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  amemiya is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with amemiya.  If not, see <http://www.gnu.org/licenses/>.
**/

#ifndef AM_FFMPEG_H_
#define AM_FFMPEG_H_

#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>

#include "AM_Boolean.h"
#include "AM_VideoContext.h";
#include "AM_AudioContext.h"
#include "AM_ArrayList.h"
#include "AM_String.h"
#include "AM_Trim.h"
#include "AM_VideoFrame.h"
#include "AM_AudioFrame.h"

typedef struct AM_Ffmpeg_Struct *AM_Ffmpeg;
AM_Ffmpeg newFfmpegContext();
struct AM_Ffmpeg_Struct{
	AVFormatContext *fc;
	AM_VideoContext vc;
	AM_AudioContext ac;
	int				readChoice;
	char 			*videoFile;
	AVPacket		decodingPacket;
	int				decodePacketFreed;
	FILE			*out;
};

void am_initAV(AVFormatContext **fc, const char *videoFile);
AM_Ffmpeg am_openVideo(char *videoFile, const char *outFile, const int resizeWidth, const int resizeHeight, const int readChoice, const AM_ArrayList videoTrims, const AM_ArrayList audioTrims);
void am_closeVideo(const AM_Ffmpeg c);
AM_Boolean am_getNextFrame(const AM_Ffmpeg c);
AM_Boolean am_readVideo(const AM_Ffmpeg c);
AM_Boolean am_decodeVideoFrame(const AM_Ffmpeg c);
AM_Boolean am_decodeAudioFrame(const AM_Ffmpeg c);
void am_seekToFirstFrame(AVFormatContext *fc, const int streamIndex);
void am_setUpFrame(const AM_Ffmpeg c);
void am_resizeFrame(const AM_VideoContext vc);
void am_printCodecString(AVCodecContext *avcc);

void avFreePacket(AVPacket *pkt);
void am_fprintVideo(const AM_Ffmpeg c);
void am_fprintAudio(const AM_Ffmpeg c);
int am_printAudio(uint8_t *outbuf, int outsize, FILE *outFile);

#endif /* AM_FFMPEG_H_ */
