/** This file is part of amemiya.
 *
 *  amemiya is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  amemiya is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with amemiya.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "AM_ArrayList.h"
#include "AM_String.h"

static void freeChar(const AM_String s, const int index){
	char *c = (char*)((am_listGet(s->chars,index)));
	//fprintf(stderr,"freeing a char\n");
	free(c);
}

static void freeChars(const AM_String s){
	int i=0;
	for(i=0; i<s->chars->size; i++){
		freeChar(s,i);
	}
}

static void freeString(const AM_String s){
	if(s->chars!=NULL){
		freeChars(s);
		am_freeList(s->chars);
	}
	if(s!=NULL){
		free(s);
	}
}

int am_stringLength(const AM_String s){
	return s->chars->size;
}

char am_stringGetChar(const AM_String s, const int index){
	//fprintf(stderr,"Trying to get char\n");
	char *c = (char*)((am_listGet(s->chars,index)));
	//fprintf(stderr,"char = %c",c[0]);
	return c[0];
}

char * am_stringGetChars(const AM_String s){
	int length = am_stringLength(s);
	char *c;
	c = malloc(sizeof(char)*length+1);
	int i=0;
	for(i=0; i<length; i++){
		c[i] = am_stringGetChar(s,i);
	}
	c[length] = '\0';
	return c;
}

void am_stringAddChar(const AM_String s, const char c){
	char *cc = malloc(sizeof(char)*2);
	cc[0]=c;
	cc[1]='\0';
	//am_listAdd(s->chars,(void *)((int)(c)));
	am_listAdd(s->chars,(void *)(cc));
}

void am_stringAddChars(const AM_String s, const char *c){
	int length = strlen(c);
	int i=0;
	for(i=0; i<length; i++){
		am_stringAddChar(s, c[i]);
	}
}

int am_stringRemoveChar(const AM_String s, const int index){
	return am_listRemove(s->chars,index);
}

int am_stringCompare(const AM_String stringOne, const AM_String stringTwo){
	if(am_stringLength(stringOne) == am_stringLength(stringTwo)){
		int i=0;
		for(i=0; i < am_stringLength(stringOne); i++){
			char a = am_stringGetChar(stringOne,i);
			char b = am_stringGetChar(stringTwo,i);
			if(a!=b){
				return 0;
			}
		}
		return 1;
	}
	return 0;
}

AM_ArrayList am_stringTokenize(const AM_String s, char delimeter){
	AM_ArrayList l = am_newArrayList();
	AM_String m = am_newString();
	int i;
	for(i=0; i<am_stringLength(s); i++){
		if(am_stringGetChar(s,i)==delimeter){
			if(i+1!=am_stringLength(s)){
				am_listAdd(l,m);
				m = am_newString();
			}
		}else{
			am_stringAddChar(m,am_stringGetChar(s,i));
		}
	}
	am_listAdd(l,m);
	return l;
}

/*static AM_String stringSearch(const AM_String s, const AM_String toSearchFor){
	if(s->length(s) < toSearchFor->length(toSearchFor)){
		return NULL;
	}else{
		int i=0;
		int searchIndex = 0;
		AM_String outAM_String = newAM_String();
		for(i=0; i<s->length(s); i++){
			if(s->getChar(s,i) == toSearchFor->getChar(toSearchFor,searchIndex)){
				if((i+(toSearchFor->length(toSearchFor))-1)>=s->length(s) && searchIndex==0){
					return NULL;
				}else if(1){

				}
				searchIndex++;
				outAM_String->addChar(outAM_String,s->getChar(s,i));
			}else{
				searchIndex = 0;
			}
		}
	}
}*/



AM_String am_newString(){
	AM_String s;
	s = malloc(sizeof(struct AM_String_Struct));
	s->chars = am_newArrayList();
	s->free=freeString;
	return s;
}
