/** This file is part of amemiya.
 *
 *  amemiya is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  amemiya is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with amemiya.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <stdlib.h>
#include <stdio.h>
#include "AM_Trim.h"
#include "AM_ArrayList.h"
#include "AM_String.h"

static AM_Trim parseToken(const AM_String s);
//void am_freeTrim(const AM_Trim trim);

int64_t am_trimTotal(const AM_Trim t){
	return (t->finish) - (t->start) + (int64_t)1;
}

void am_freeTrim(const AM_Trim trim){
	if(trim!=NULL){
		free(trim);
	}
}

void am_parseTrimString(const AM_ArrayList trimList, const AM_String s){
	int i=0;
	for(i=0; i<5; i++){
		am_stringRemoveChar(s,0);
	}
	am_stringRemoveChar(s,am_stringLength(s)-1);
	AM_ArrayList tokens = am_stringTokenize(s,',');
	for(i=0; i<tokens->size; i++){
		am_listAdd(trimList,parseToken(am_listGet(tokens,i)));
	}
	am_freeList(tokens);
}


static AM_Trim parseToken(const AM_String s){
	AM_ArrayList l = am_stringTokenize(s,'-');
	AM_String st = (AM_String)am_listGet(l,0);
	char *start = am_stringGetChars(st);
	int64_t startNum = (int64_t)atoi(start);
	AM_String en = (AM_String)am_listGet(l,1);
	char *end = am_stringGetChars(en);
	int64_t endNum = (int64_t)atoi(end);
	st->free(st);
	en->free(en);
	am_freeList(l);
	s->free(s);
	free(start);
	free(end);
	return am_newTrim(startNum,endNum);
}

static int64_t am_trimCalculateAudioFrame(const int64_t frameNumber, const int64_t totalVideoFrames, const int64_t totalAudioFrames){
	int64_t f = frameNumber;
	int64_t v = totalVideoFrames;
	int64_t a = totalAudioFrames;
	int64_t n = (int64_t)(f*((double)a/(double)v));
	return n;
}

void am_createAudioTrims(const AM_ArrayList videoTrim, const AM_ArrayList audioTrim, const int64_t totalVideoFrames, const int64_t totalAudioFrames){
	int iterator;
	for(iterator=0; iterator<videoTrim->size; iterator++){
		int64_t start = am_trimCalculateAudioFrame(((AM_Trim)am_listGet(videoTrim,iterator))->start,totalVideoFrames,totalAudioFrames);
		int64_t finish = am_trimCalculateAudioFrame(((AM_Trim)am_listGet(videoTrim,iterator))->finish,totalVideoFrames,totalAudioFrames);
		am_listAdd(audioTrim,am_newTrim(start,finish));
	}
}

int am_insideTrim(const AM_Trim t, const int64_t i){
	if(i>=(t->start)&&i<=(t->finish)){
		return 1;
	}
	return 0;
}

int am_atTrimEnd(const AM_Trim t, const int64_t i){
	if(i==(t->finish)){
		return 1;
	}
	return 0;
}

void am_printTrim(const AM_Trim t){
	fprintf(stderr,"trim(%lld-%lld)\n",t->start,t->finish);
}

AM_Trim am_newTrim(const int64_t _start, const int64_t _finish){
	AM_Trim t;
	t = malloc(sizeof(struct AM_Trim_Struct));
	if(t == NULL){
		fprintf(stderr, "Out of memory\n");
		fflush(stderr);
		exit(0);
	}
	t->start=_start;
	t->finish=_finish;
	return t;
}
