/*
 * amemiya A video and audio frame server.
 * Copyright (C) <2010>  Tate Exon
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>

#include "AM_Boolean.h"
#include "AM_VideoFrame.h"
#include "AM_VideoContext.h"
#include "AM_AudioFrame.h"
#include "AM_AudioContext.h"
#include "AM_ArrayList.h"
#include "AM_String.h"
#include "AM_Trim.h"
#include "AM_Ffmpeg.h"

void outFormatStuff(AVFormatContext *avfc);

int main (int argc, const char **argv){
	fprintf(stderr,"amemiya  Copyright (C) 2010 Tate Exon\n");
	fprintf(stderr,"This program comes with ABSOLUTELY NO WARRANTY;\n");
	fprintf(stderr,"This is free software, and you are welcome to redistribute it\n");
	fprintf(stderr,"under certain conditions; License: GPL Version 3\n");

	AM_Ffmpeg 		c;
	char 			videoFile[1024];
	char 			outputFile[1024];
	int 			optionChoice=0;
	int 			wantedWidth=0;
	int 			wantedHeight=0;
	char 			trimString[2024];
	int64_t			totalVideoFrames=0;
	int64_t			totalAudioFrames=0;
	AM_ArrayList	videoTrims=am_newArrayList();
	AM_ArrayList	audioTrims=am_newArrayList();

	if(argc < 3){
		fprintf(stderr,"not enough arguments!\n");
		return 0;
	}else if(argc > 2){
		sprintf(videoFile,"%s",argv[1]);
		sprintf(outputFile,"%s",argv[2]);
		optionChoice = atoi(argv[3]);
		if(optionChoice != 4){
			if(argc > 4 && optionChoice == 1){
				wantedWidth 	= atoi(argv[4]);
				wantedHeight 	= atoi(argv[5]);
				if(argc > 5){
					sprintf(trimString,"%s",argv[6]);
					AM_String string = am_newString();
					am_stringAddChars(string,trimString);
					am_parseTrimString(videoTrims,string);
					string->free(string);
				}
			}else if(argc > 4 && optionChoice == 2){
				if(argc == 7){
					sprintf(trimString,"%s",argv[4]);
					AM_String string = am_newString();
					am_stringAddChars(string,trimString);
					am_parseTrimString(videoTrims,string);
					fprintf(stderr,"freeing string\n");
					string->free(string);
					totalVideoFrames = (int64_t)atoi(argv[5]);
					totalAudioFrames = (int64_t)atoi(argv[6]);
					am_createAudioTrims(videoTrims, audioTrims, totalVideoFrames, totalAudioFrames);
				}else{
					fprintf(stderr,"Cannot trim audio without 6 arguments:\n");
					fprintf(stderr,"videofile, outfile, option choice, trim string,\n");
					fprintf(stderr,"total video frames, total audio frames");
					return 0;
				}
			}
		}
	}

	// uncomment to see what the trims are.
	/*int iter = 0;
	fprintf(stderr,"video trims = ");
	for(iter=0;iter<videoTrims->size; iter++){
		int64_t s = ((AM_Trim)am_listGet(videoTrims,iter))->start;
		int64_t f = ((AM_Trim)am_listGet(videoTrims,iter))->finish;
		fprintf(stderr,"%lld-%lld, ",s,f);
	}
	fprintf(stderr,"\n");
	fprintf(stderr,"audio trims = ");
	for(iter=0;iter<audioTrims->size; iter++){
		int64_t s = ((AM_Trim)am_listGet(audioTrims,iter))->start;
		int64_t f = ((AM_Trim)am_listGet(audioTrims,iter))->finish;
		fprintf(stderr,"%lld-%lld, ",s,f);
	}
	fprintf(stderr,"\n");
	*/

	c = am_openVideo(videoFile,outputFile,wantedWidth,wantedHeight,
			optionChoice,videoTrims,audioTrims);
	am_readVideo(c);
	am_closeVideo(c);
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);
    return 1;
}

void outFormatStuff(AVFormatContext *avfc){
	if(avfc->duration != AV_NOPTS_VALUE){
		int hours,min,secs,us;
		secs = avfc->duration / AV_TIME_BASE;
		us = avfc->duration % AV_TIME_BASE;
		min = secs / 60;
		secs %= 60;
		hours = min / 60;
		min %= 60;
		fprintf(stderr,"duration = %d:%d:%d.%d\n",hours,min,secs,us);
		int j=0;
		for(j=0; j<avfc->nb_streams; j++){
			AVStream *st = avfc->streams[j];
			fprintf(stderr,"number of frames = %lld\n",st->nb_frames);
		}
	}
}

/*static int findAudioBytePerSecond(int hz, int channels){
	return hz*channels*2;
}

static int findAudioByteStartFromVideoFrame(int currentVideoFrame, double videoFps, int audioBytesPerSecond){
	return (long)(((double)currentVideoFrame/(double)videoFps)*(audioBytesPerSecond));
}

static int findAudioFrameWithStartByte(long startByte, int bytesPerFrame){
	long bpf = (long)bytesPerFrame;
	return startByte/bpf;
}

static int findAudioOffsetAmount(int frame, int bpf, long startByte){
	long lframe = (long)frame;
	long lbpf = (long)bpf;
	long tot = (long)(lframe*lbpf);
	return (int)(startByte-tot);
}

static uint8_t * getPartialAudioFrame(uint8_t *frame, int frameSize, int startNum){
	if((startNum % 4) != 0){
		startNum+=1;
	}
	if((startNum % 4) !=0){
		startNum+=1;
	}
	if((startNum % 4) !=0){
		startNum-=3;
	}
	uint8_t *newTmp = malloc(sizeof(uint8_t)*(frameSize-startNum));
	int i;
	for(i=startNum; i<frameSize; i++){
		newTmp[i-startNum] = frame[i];
	}
	return newTmp;
}
*/
